/****************************************************************************
 *                                                                          *
 * File    : DbgRelPrf.c                                                    *
 *                                                                          *
 * Purpose : Sample add-in DLL. Switch between Debug, Release and           *
 *           Profile build.                                                 *
 *                                                                          *
 * History : Date      Reason                                               *
 *           04-02-04  Created (Pelle Orinius)                              *
 *           04-02-15  Added SetAssemblerFlags() - for MASM (Pelle Orinius) *
 *           04-11-25  Added Profile build option (Alexander Stoica)        *
 ****************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include <addin.h>
#include "DbgRelPrf.h"

#define NELEMS(a)  (sizeof(a) / sizeof(a[0]))

// Private command IDs - any numbers will do.
#define ID_RELEASE 1
#define ID_DEBUG 2
#define ID_PROFILE 3

// Locals.
static HANDLE g_hmod = NULL;
static HWND g_hwndMain = NULL;
static HWND g_hwndPrj = NULL;

static BOOL SetCompilerFlags(int idCmd);
static BOOL SetAssemblerFlags(int idCmd);
static BOOL SetLinkerFlags(int idCmd);

/****************************************************************************
 *                                                                          *
 * Function: DllMain                                                        *
 *                                                                          *
 * Purpose : DLL entry and exit procedure.                                  *
 *                                                                          *
 * History : Date      Reason                                               *
 *           00/00/00  Created                                              *
 *                                                                          *
 ****************************************************************************/

BOOL WINAPI DllMain(HANDLE hDLL, DWORD dwReason, LPVOID lpReserved)
{
    switch (dwReason)
    {
        case DLL_PROCESS_ATTACH:
            g_hmod = hDLL;
            return TRUE;

        case DLL_PROCESS_DETACH:
            return TRUE;

        default:
            return TRUE;
    }
}

/****************************************************************************
 *                                                                          *
 * Function: AddInMain                                                      *
 *                                                                          *
 * Purpose : Add-in entry and exit procedure.                               *
 *                                                                          *
 * History : Date      Reason                                               *
 *           00/00/00  Created                                              *
 *                                                                          *
 ****************************************************************************/

ADDINAPI BOOL WINAPI AddInMain(HWND hwnd, ADDIN_EVENT eEvent)
{
    switch (eEvent)
    {
        case AIE_APP_CREATE:
            /* Save handle of main window */
            g_hwndMain = hwnd;
            return TRUE;

        case AIE_PRJ_CREATE:
        {
            ADDIN_ADD_COMMAND AddCmd = {0};

            /* Save handle of project window */
            g_hwndPrj = hwnd;

            /* Add command buttons to project toolbar */
            AddCmd.cbSize = sizeof(AddCmd);
            AddCmd.pszText = "Release build";  // Tooltip text.
            AddCmd.hIcon = LoadImage(g_hmod, MAKEINTRESOURCE(IDR_RELEASE), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR|LR_SHARED);  // Button icon.
            AddCmd.id = ID_RELEASE;
            AddCmd.idMenu = AIM_MENU_NOTHING;
            if (!AddIn_AddCommand(hwnd, &AddCmd))
                return FALSE;

            AddCmd.pszText = "Debug build";  // Tooltip text.
            AddCmd.hIcon = LoadImage(g_hmod, MAKEINTRESOURCE(IDR_DEBUG), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR|LR_SHARED);  // Button icon.
            AddCmd.id = ID_DEBUG;
            if (!AddIn_AddCommand(hwnd, &AddCmd))
                return FALSE;

            AddCmd.pszText = "Profile build";  // Tooltip text.
            AddCmd.hIcon = LoadImage(g_hmod, MAKEINTRESOURCE(IDR_PROFILE), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR|LR_SHARED);  // Button icon.
            AddCmd.id = ID_PROFILE;
            if (!AddIn_AddCommand(hwnd, &AddCmd))
                return FALSE;

            return TRUE;
        }

        case AIE_PRJ_DESTROY:
            /* Remove buttons from project toolbar */
            return AddIn_RemoveCommand(hwnd, ID_RELEASE) && 
                   AddIn_RemoveCommand(hwnd, ID_DEBUG) &&
                   AddIn_RemoveCommand(hwnd, ID_PROFILE);

        default:
            return TRUE;
    }
}

/****************************************************************************
 *                                                                          *
 * Function: AddInCommandEx                                                 *
 *                                                                          *
 * Purpose : Add-in command handler.                                        *
 *                                                                          *
 * History : Date      Reason                                               *
 *           00/00/00  Created                                              *
 *                                                                          *
 ****************************************************************************/

ADDINAPI void WINAPI AddInCommandEx(int idCmd, LPCVOID pcvData)
{
    SetCompilerFlags(idCmd);
    SetAssemblerFlags(idCmd);
    SetLinkerFlags(idCmd);
    AddIn_SendIDECommand(g_hwndMain, AIC_PROJ_BUILDALL);
}

/****************************************************************************
 *                                                                          *
 * Function: SetCompilerFlags                                               *
 *                                                                          *
 * Purpose : Set CCFLAGS to debug, release or profile build.                *
 *                                                                          *
 * History : Date      Reason                                               *
 *           04-02-04  Created                                              *
 *                                                                          *
 ****************************************************************************/

static BOOL SetCompilerFlags(int idCmd)
{
    TCHAR szOld[1024], szNew[1024], *psz;

    /* Get the current compiler options */
    if (AddIn_GetProjectSymbol(g_hwndPrj, _T("CCFLAGS"), szOld, NELEMS(szOld)) == 0)
        return FALSE;

    /* Set the new options */
    switch (idCmd)
    {
        case ID_RELEASE:
        {
            lstrcpy(szNew, _T(""));
            break;
        }
        case ID_DEBUG:
        {
            lstrcpy(szNew, _T("-Zi"));
            break;
        }
        case ID_PROFILE:
        {
            lstrcpy(szNew, _T("-Zi -Gh"));
            break;
        }
    }

    /* Walk through the current options and build the new options */
    for (psz = _tcstok(szOld, _T(" ")); psz != NULL; psz = _tcstok(NULL, _T(" ")))
    {
        /* Forget old debug and profile options */
        if (lstrcmp(psz, _T("-Zi")) == 0 || lstrcmp(psz, _T("-Zd")) == 0 || lstrcmp(psz, _T("-Gh")) == 0)
            continue;

        /* Add this option to the new compiler options */
        if (szNew[0] != '\0')
            lstrcat(szNew, _T(" "));
        lstrcat(szNew, psz);
    }

    /* Set the new compiler options */
    return AddIn_SetProjectSymbol(g_hwndPrj, _T("CCFLAGS"), szNew);
}

/****************************************************************************
 *                                                                          *
 * Function: SetAssemblerFlags                                              *
 *                                                                          *
 * Purpose : Set ASFLAGS to debug, release or profile build.                *
 *                                                                          *
 * History : Date      Reason                                               *
 *           04-02-15  Created                                              *
 *                                                                          *
 ****************************************************************************/

static BOOL SetAssemblerFlags(int idCmd)
{
    TCHAR szOld[1024], szNew[1024], *psz;

    /* Get the current assembler options */
    if (AddIn_GetProjectSymbol(g_hwndPrj, _T("ASFLAGS"), szOld, NELEMS(szOld)) == 0)
        return FALSE;

    /* Set the new options */
    switch (idCmd)
    {
        case ID_RELEASE:
        {
            lstrcpy(szNew, _T(""));
            break;
        }
        case ID_DEBUG:
        {
            lstrcpy(szNew, _T("-Zi"));
            break;
        }
        case ID_PROFILE:
        {
            lstrcpy(szNew, _T("-Zi -Gh"));
            break;
        }
    }

    /* Walk through the current options and build the new options */
    for (psz = _tcstok(szOld, _T(" ")); psz != NULL; psz = _tcstok(NULL, _T(" ")))
    {
        /* Forget old debug and profile options */
        if (lstrcmp(psz, _T("-Zi")) == 0 || lstrcmp(psz, _T("-Zd")) == 0 || lstrcmp(psz, _T("-Gh")) == 0)
            continue;

        /* Add this option to the new compiler options */
        if (szNew[0] != '\0')
            lstrcat(szNew, _T(" "));
        lstrcat(szNew, psz);
    }

    /* Set the new compiler options */
    return AddIn_SetProjectSymbol(g_hwndPrj, _T("ASFLAGS"), szNew);
}

/****************************************************************************
 *                                                                          *
 * Function: SetLinkerFlags                                                 *
 *                                                                          *
 * Purpose : Set LINKFLAGS to debug, release or profile build.              *
 *                                                                          *
 * History : Date      Reason                                               *
 *           04-02-04  Created                                              *
 *                                                                          *
 ****************************************************************************/

static BOOL SetLinkerFlags(int idCmd)
{
    TCHAR szOld[1024], szNew[1024], *psz;

    /* Get the current linker options */
    if (AddIn_GetProjectSymbol(g_hwndPrj, _T("LINKFLAGS"), szOld, NELEMS(szOld)) == 0)
        return FALSE;

    /* Set the new options */
    switch (idCmd)
    {
        case ID_RELEASE:
        {
            lstrcpy(szNew, _T("-release"));
            break;
        }
        case ID_DEBUG:
        {
            lstrcpy(szNew, _T("-debug -debugtype:both"));
            break;
        }
        case ID_PROFILE:
        {
            if (_tcsstr(szOld, _T("-subsystem:console")) == NULL)
            {
                lstrcpy(szNew, _T("-debug -debugtype:both profiler.lib"));
            }
            else
            {
                lstrcpy(szNew, _T("-debug -debugtype:both profiler.lib user32.lib"));
            }

            break;
        }
    }

    if (_tcsstr(szOld, _T("-subsystem:console")) == NULL)
    {
        /* Walk through the current options and build the new options */
        for (psz = _tcstok(szOld, _T(" ")); psz != NULL; psz = _tcstok(NULL, _T(" ")))
        {
            /* Forget old options */
            if (lstrcmp(psz, _T("-release")) == 0 || 
                lstrcmp(psz, _T("-debug")) == 0 || 
                lstrcmp(psz, _T("-debugtype:both")) == 0 ||
                lstrcmp(psz, _T("-debugtype:coff")) == 0 ||
                lstrcmp(psz, _T("-debugtype:cv")) == 0 ||
                lstrcmp(psz, _T("profiler.lib")) == 0)
                continue;

            /* Add this option to the new linker options */
            if (szNew[0] != '\0')
                lstrcat(szNew, _T(" "));
            lstrcat(szNew, psz);
        }
    }
    else
    {
        /* Walk through the current options and build the new options */
        for (psz = _tcstok(szOld, _T(" ")); psz != NULL; psz = _tcstok(NULL, _T(" ")))
        {
            /* Forget old options */
            if (lstrcmp(psz, _T("-release")) == 0 || 
                lstrcmp(psz, _T("-debug")) == 0 || 
                lstrcmp(psz, _T("-debugtype:both")) == 0 ||
                lstrcmp(psz, _T("-debugtype:coff")) == 0 ||
                lstrcmp(psz, _T("-debugtype:cv")) == 0 ||
                lstrcmp(psz, _T("profiler.lib")) == 0 ||
                ((idCmd != ID_PROFILE) && (lstrcmp(psz, _T("user32.lib")) == 0)))
                continue;

            /* Add this option to the new linker options */
            if (szNew[0] != '\0')
                lstrcat(szNew, _T(" "));
            lstrcat(szNew, psz);
        }
    }

    /* Set the new linker options */
    return AddIn_SetProjectSymbol(g_hwndPrj, _T("LINKFLAGS"), szNew);
}
